// Wallpaper position data
var _center_x = obj_camera.wallpaper_width * 0.5;
var _center_y = obj_camera.wallpaper_height * 0.5;

var _curr_display_width = camera_get_view_width(view_camera[0]);
var _curr_display_height = camera_get_view_height(view_camera[0]);

var _origin_x = _center_x - (_curr_display_width * 0.5);
var _origin_y = _center_y - (_curr_display_height * 0.5);

var _floor_level = _origin_y + _curr_display_height;

// Checks for audio bars
if (instance_exists(obj_audio_bar))
{
	// Adjusts the floor level from the audio visualiser settings 
	_floor_level -=	obj_audio_visualiser.min_height;
}

// Sets the new momentum clampeds
var _new_momentum = clamp(angle_momentum * 0.1, -30, 30);

// Checks held state
if (is_held)
{	
	// Stops movement speed
	hspeed = 0;
	vspeed = 0;

	// Offsets for position
	var _mouse_origin_x = mouse_x - x;
	var _mouse_origin_y = mouse_y - y;

	// New angle stored in radians
	var _theta = degtorad(-_new_momentum);

	// Calculates the adjusted positions from offsets and angle
	var _mouse_adjust_x = (_mouse_origin_x * cos(_theta)) - (_mouse_origin_y * sin(_theta));
	var _mouse_adjust_y = (_mouse_origin_y * cos(_theta)) + (_mouse_origin_x * sin(_theta));

	// Sets new postions from adjusted positions
	var _new_mouse_pos_x = x + _mouse_adjust_x;
	var _new_mouse_pos_y = y + _mouse_adjust_y;

	// Adjusts the position
	x += mouse_x - _new_mouse_pos_x;
	y += mouse_y - _new_mouse_pos_y;
}
else
{
	// Checks is in air
	if (!is_grounded)
	{
		// Fall
		vspeed += 0.5;
	}
	
	// Slow horizontal movement
	hspeed *= 0.9;
}

// Adjust image angle from momentum
image_angle += _new_momentum;

// Slows the angles momentum
angle_momentum *= 0.9;

// Limits the max speed
speed = min(speed, 9);

// Checks if the object has left the play area to the left and right boundries
if (bbox_right < _origin_x)
{
	// Destroys the object
	instance_destroy();
}
else if (bbox_left > _origin_x + _curr_display_width)
{
	// Destroys the object
	instance_destroy();
}

// Checks if the object has reached the floor
if (bbox_bottom > _floor_level - 5)
{
	// Calculates the difference off screen
	var _diff = bbox_bottom - (_floor_level);
	
	// Adjusts position
	y -= _diff;
	
	// Slows the speed
	vspeed = -abs(vspeed) * 0.5;
	
	// Calculates the images closest facing in full rotatations
	var _image_face = round((image_angle % 360) / 360);
	var _target_face = _image_face * 360;
	
	// Adjust the angle momentum towards nearest resting face
	angle_momentum = angle_difference(image_angle % 360, lerp(image_angle % 360, (_target_face), 0.5));
	
	// Adjusts image angle to target
	image_angle = lerp(image_angle % 360, (_target_face), 0.1);
	
	// Slows the horizontal speed
	hspeed *= 0.5;
	
	// Sets the grounded state
	is_grounded = true;
}
else
{
	// Unsets the grounded state
	is_grounded = false;
}

// Updates previous mouse positions
prev_mouse_x = mouse_x;
prev_mouse_y = mouse_y;