// Checks if disk metrics exists
if (obj_lw_handler.disk_exist)
{
	// Set variables for disk usage
	var _disk_used = 0;
	var _disk_total = 0;
	
	// Loop through available disks
	for (var _i = 0; _i < obj_lw_handler.disk_count; _i++)
	{
		// Add disk values to count
		_disk_used += obj_lw_handler.disk_used[_i];
		_disk_total += obj_lw_handler.disk_total[_i];
	}
	
	// Create min variable based on screen size (third of screen)
	var _curr_display_min = camera_get_view_height(view_camera[0]) * (1 / 3);
	
	// Calculate min height as a percentage of used disk space
	min_height = _curr_display_min * (_disk_used / _disk_total);
	
	// Cap min height to never be less than 100
	min_height = max(min_height, 100);
}
else
{
	// Sets default min height of 100
	min_height = 100;
}

// Set max height from adjusted min height
max_height = min_height + 100;

// Checks if audio exists
if (obj_lw_handler.audio_exist)
{
	// Create variables used to store values for wave form cooridinates
	var _point_coords = [0, 0];
	var _wave_points = array_create(real(staged_audio_length), _point_coords);

	// Wallpaper position data
	var _center_x = obj_camera.wallpaper_width * 0.5;
	var _center_y = obj_camera.wallpaper_height * 0.5;

	var _curr_display_width = camera_get_view_width(view_camera[0]);
	var _curr_display_height = camera_get_view_height(view_camera[0]);

	var _origin_x = _center_x - (_curr_display_width * 0.5);
	var _origin_y = _center_y - (_curr_display_height * 0.5);
	
	// Works out max positions for display
	var _max_x = _origin_x + _curr_display_width;
	var _max_y = _origin_y + _curr_display_height;

	// Variable for water level (lets the bottoms of object look submerged)
	var _y_adjustment = 20;

	// Loops through water segments
	with (obj_water_segment)
	{
		// Saves their coordinates with their id
		_wave_points[staged_id] = [x, y];	
	}
	
	// Resize the surface
	surface_resize(underwater_surface, _max_x, _max_y);
	
	// Set the surface as render target
	surface_set_target(underwater_surface);
	
	// Creates custom colour
	var _col_dark = make_color_rgb(45, 154, 228);
	
	// Draw the background image using max values as dimensions with a gradient and transparency
	draw_sprite_general(spr_background_tub, 0, 0, 0, _max_x, _max_y, 0, 0, 1, 1, 0, c_white, c_white, _col_dark, _col_dark, 0.5);
	
	// Reset the surface render target
	surface_reset_target();
	
	// Set draw and texture options for waveform from surface
	draw_set_colour(c_white);
	var _tex = surface_get_texture(underwater_surface);
	
	// Loops through the audio values stored
	if (staged_audio_length > 0)
	{
		// Create primative shape using the texture set
		draw_primitive_begin_texture(pr_trianglestrip, _tex);
	
		// Create inital point as bottom left of screen
		draw_vertex_texture(_origin_x, _origin_y + _curr_display_height, _origin_x / _max_x, (_origin_y + _curr_display_height) / _max_y);

		// Loop for all but last points available
		for (var _i = 0; _i < staged_audio_length - 1; _i++)
		{	
			// Create vertex group for each set of points
			draw_vertex_texture(_wave_points[_i][0], _wave_points[_i][1] - _y_adjustment, _wave_points[_i][0] / _max_x, (_wave_points[_i][1] - _y_adjustment) / _max_y);
			draw_vertex_texture(_wave_points[_i + 1][0], _wave_points[_i + 1][1] - _y_adjustment, _wave_points[_i + 1][0] / _max_x, (_wave_points[_i + 1][1] - _y_adjustment) / _max_y);
			draw_vertex_texture(_wave_points[_i][0], _origin_y + _curr_display_height, _wave_points[_i][0] / _max_x, (_origin_y + _curr_display_height) / _max_y);
			draw_vertex_texture(_wave_points[_i + 1][0], _origin_y + _curr_display_height, _wave_points[_i + 1][0] / _max_x, (_origin_y + _curr_display_height) / _max_y);
		}
		
		// Draw last point of sets
		draw_vertex_texture(_wave_points[staged_audio_length - 1][0], _wave_points[staged_audio_length - 1][1] - _y_adjustment, _wave_points[staged_audio_length - 1][0] / _max_x, (_wave_points[staged_audio_length - 1][1] - _y_adjustment) / _max_y);
		
		// Calculated final point
		var _final_y = ((_wave_points[staged_audio_length - 1][1] - _y_adjustment) + ( _wave_points[0][1] - _y_adjustment)) * 0.5;
		
		// Final group of points
		draw_vertex_texture(_origin_x + _curr_display_width, _final_y, (_origin_x + _curr_display_width) / _max_x, _final_y / _max_y);
		draw_vertex_texture(_wave_points[staged_audio_length - 1][0], _origin_y + _curr_display_height, _wave_points[staged_audio_length - 1][0] / _max_x, (_origin_y + _curr_display_height) / _max_y);
		draw_vertex_texture(_origin_x + _curr_display_width, _origin_y + _curr_display_height , (_origin_x + _curr_display_width) / _max_x, (_origin_y + _curr_display_height) / _max_y);
		draw_vertex_texture(_origin_x + _curr_display_width, _final_y, (_origin_x + _curr_display_width) / _max_x, _final_y / _max_y);
		
		// End primative draw
		draw_primitive_end();
	}
	
	// Set new colour for top of wave
	draw_set_colour(make_color_rgb(156,212,248));
	
	// Loop through audio again
	if (staged_audio_length > 0)
	{
		// Draw only from colour new primative
		draw_primitive_begin(pr_trianglestrip);
		
		// Create inital point
		draw_vertex(_wave_points[0][0], _wave_points[0][1] - _y_adjustment);
	
		// Loop for all but last segments
		for (var _i = 0; _i < staged_audio_length - 1; _i++)
		{	
			// Draw only one point for each wave
			draw_vertex(_wave_points[_i + 1][0], _wave_points[_i + 1][1] - _y_adjustment);
		}
	
		// Draw last point
		draw_vertex(_wave_points[staged_audio_length - 1][0], _wave_points[staged_audio_length - 1][1] - _y_adjustment);
		
		// End draw
		draw_primitive_end();
	}
	
	// Reset colour
	draw_set_colour(c_white);
}